/* This file is part of tagr.
   Copyright (C) 2000, 2005,  Max Bouglacoff, Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA. */

#include <time.h>
#include <errno.h>
#include <xalloc.h>

#define LOGFACILITY LOG_LOCAL5
#define TAGR_CONFIGFILE SYSCONFDIR "/tagr.conf"
#define TAGR_TEMPLATE SYSCONFDIR "/tagr.tmpl"
#define TAGR_PIDFILE "/var/run/tagr.pid"
#define TAGR_DBNAME "tagr.db"
#define TAGR_DBMODE 0600

typedef struct HostData
{
  char *id;
  char *name;
  char *dir;
  time_t t;
  unsigned in;
  unsigned out;
  unsigned max;
} SD;

extern char rateup[];
extern char *user;
extern char *basedir;
extern char *configfile;
extern int port;
extern char *progname;
extern int foreground;
extern char *html_template;
extern char *html_input_file;
extern int html_input_line;
extern int verbose_level;

extern int percent_option;
extern int transparent_option;
extern int zero_unknown_option;
extern int fill_incoming_option;
extern int color_background[3];
extern int color_light[3];
extern int color_dark[3];
extern int color_major[3];
extern int color_in[3];
extern int color_out[3];
extern int color_grid[3];
extern int color_in_max[3];
extern int color_out_max[3];
extern int color_percent[3];
extern char **number_suffix;
extern size_t number_suffix_count;

void assign_string (char **pstr, const char *s);
void assign_string_n (char **pstr, const char *s, size_t length);

int readconfig (void);
void decode_buffer ();
SD *find_router (const char *name);
SD *find_router_id (const char *id);

void begin_eval ();
void end_eval ();
int html_open (char *file);
void html_close ();
int yyerror (char *s);

typedef enum { unspecified_value, numeric_value, string_value } value_type;

typedef struct
{
  value_type type;
  int prec;
  union
  {
    double number;
    char *string;
  } v;
} pp_value_t;

typedef struct pp_value_list
{
  struct pp_value_list *next;
  char *name;
  pp_value_t value;
} pp_tab_t;

void add_numeric_value (pp_tab_t ** ptab, const char *name, double number);
void add_string_value (pp_tab_t ** ptab, const char *name, const char *string);
int find_value (char *name, pp_value_t * val);
void free_tab (pp_tab_t ** ptab);
int create_html (pp_tab_t * tab, char *file, char *dest);
int check_template ();

/* Logging */
#define L_DEBUG   0
#define L_INFO    1
#define L_NOTICE  2
#define L_WARNING 3
#define L_ERR     4
#define L_CRIT    5

extern int use_stderr;

void init_syslog (char *name);
void logmsg (int level, const char *fmt, ...);
void die (int code, const char *fmt, ...);
void verbose (int level, const char *fmt, ...);



/* Traffic history */

#define DAY_COUNT (600)		/* 400 samples is 33.33 hours */
#define DAY_SAMPLE (5*60)	/* Sample every 5 minutes */
#define WEEK_COUNT (600)	/* 400 samples is 8.33 days */
#define WEEK_SAMPLE (30*60)	/* Sample every 30 minutes */
#define MONTH_COUNT (600)	/* 400 samples is 33.33 days */
#define MONTH_SAMPLE (2*60*60)	/* Sample every 2 hours */
#define YEAR_COUNT  (2 * 366)	/* 1 sample / day, 366 days, 2 years */
#define YEAR_SAMPLE (24*60*60)	/* Sample every 24 hours */

/* One 'rounding error' per sample period, so add 4 to total */
#define MAX_HISTORY (DAY_COUNT+WEEK_COUNT+MONTH_COUNT+YEAR_COUNT+4+4)

struct traffic_history
{
  double inrate;
  double outrate;
};

typedef struct stat_queue queue_t;
struct stat_queue
{
  int head;
  int tail;
  int size;
  struct traffic_history *queue;
};

struct last_sample
{
  time_t time;
  unsigned long in, out;
};

struct avg_acc
{
  time_t time;
  unsigned count;
  double inrate;
  double outrate;
};
  
struct traffic_record
{
  struct last_sample last; /* Lastly taken sample */
  struct traffic_history last_rates;
  /* Average accumulators */
  struct avg_acc week_avg;  
  struct avg_acc month_avg; 
  struct avg_acc year_avg;
  /* History arrays */
  queue_t day_hist;
  queue_t week_hist;
  queue_t month_hist;
  queue_t year_hist;
  struct traffic_history history[MAX_HISTORY];
};



void open_db ();
void close_db ();
void read_db (SD *sd, struct traffic_record **tr);
void write_db (SD *sd, struct traffic_record *tr);
void update_stats (SD *sd, struct traffic_record *tr);

void rebuild (void);
void import (const char *dir);

void queue_clear (queue_t *q);
int queue_count (queue_t *q);
struct traffic_history *queue_get_ptr (queue_t *q, int index);
struct traffic_history *queue_get_tail (queue_t *q);
void queue_put (queue_t *q, struct traffic_history *elt);
int queue_xchg (queue_t *q, struct traffic_history *elt);

char *mkfilename (char *dir, char *name, char *suffix);
